/*
 * main.c
 *
 * Created: 11/7/2014
 * Author: jiri stepanovsky, jiristepanovsky.cz
 */

#define F_CPU 4000000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "display.h"

// FUSES
// avrdude -p m8 -c usbasp -U lfuse:r:-:h
// avrdude -p m8 -c usbasp -U lfuse:w:0xef:m


volatile uint8_t segment[5];
volatile uint8_t minute = 9, second = 60, frames = 24, milisec = 0;
volatile uint8_t counting = 1, restart = 0;
uint8_t pressed = 0, pressedCounter = 0, releasedCounter = 0;
uint8_t redCable = 1, blueCable = 1, display = 0, endOfTimeDelay = 0;



ISR(TIMER1_COMPA_vect) // every 1/40s
{
	if(counting){
		// if countdown is running
		if(redCable){
			// if red cable is not cut

			// countdown
			if(frames > 0){
				frames--;

				// slow countdown before it reach absolute zero
				/*if(minute == 0 && second == 0){
					OCR1A += 399;
				} else if(minute == 0 && second < 5){
					OCR1A += 199;
				} else if(minute == 0 && second < 10){
					OCR1A += 99;
				}*/
			}else{
				if(second == 0 && minute == 0){
					// absolute zero
					counting = 0;
				} else {
					frames = 24;
			
					// seconds
					if(second > 0){
						second--;
					}else{
						second = 59;

						// minutes
						if(minute > 0){
							minute--;
						}else{
							//counting = 0;
						}
						// update minutes
						segment[0] = minute;
					}
				}
				// update seconds
				segment[1] = second/10;
				segment[2] = second%10;
			}
			// update frames
			segment[3] = frames/10;
			segment[4] = frames%10;
		} else {
			// write save
			endOfTimeDelay++;
			if(endOfTimeDelay > 159) endOfTimeDelay = 120;
			if(endOfTimeDelay == 120) {
				segment[0] = 12;
				segment[1] = 12;
				segment[2] = 12;
				segment[3] = 12;
				segment[4] = 12;
			} else if (endOfTimeDelay == 139) {
				segment[0] = 12;
				segment[1] = 5;
				segment[2] = 14;
				segment[3] = 15;
				segment[4] = 16;
			}
		}
	} else {
		// delay before countdown can be restarted with button
		//if(restart < 255)
		//	restart++;
		
		//if(!blueCable){
			// write LOL
			endOfTimeDelay++;
			if(endOfTimeDelay > 69) endOfTimeDelay = 40;
			if(endOfTimeDelay == 40) {
				segment[0] = 12;
				segment[1] = 12;
				segment[2] = 12;
				segment[3] = 12;
				segment[4] = 12;
			} else if (endOfTimeDelay == 54) {
				segment[0] = 12;
				segment[1] = 12;
				segment[2] = 13;
				segment[3] = 0;
				segment[4] = 13;
			}
		/*} else {
			// blink with zeros
			endOfTimeDelay++;
			if(endOfTimeDelay > 59) endOfTimeDelay = 40;
			if(endOfTimeDelay == 40) {
				segment[0] = 12;
				segment[1] = 12;
				segment[2] = 12;
				segment[3] = 12;
				segment[4] = 12;
			} else if (endOfTimeDelay == 49) {
				segment[0] = 0;
				segment[1] = 0;
				segment[2] = 0;
				segment[3] = 0;
				segment[4] = 0;
			}
		}*/
	}
}

int main(void)
{

	// set time
	minute = 9;
	second = 59;
	frames = 24;
	segment[0] = minute;
	segment[1] = second/10;
	segment[2] = second%10;
	segment[3] = frames/10;
	segment[4] = frames%10;

	// set timer in CRC mode
	TCCR1B |= (1 << WGM12) | (1 << CS11);
	TCNT1 = 0;
	OCR1A = 19999; // create 1/40s delay
	TIMSK |= (1 << OCIE1A);

	// enable interrupts
	sei();

	// initialize ports direction
	HW_init();

	// if button is pressed
	if(!(PIND&b00010000)) pressed = 1;
	
	// if any of wire is connected
	if(!(PINC&b00100000)){
		redCable = 2;
	}
	if(!(PINC&b00010000)){
		blueCable = 2;
	}

	while(1){
		// infinite delay


		// show individualy all segments with 2.5ms delay for all
		_delay_us(500);
		clear();
		setSegments(segment[display]);
		setDisplay(display);
		display++;
		if(display > 4) display = 0;

		// check button state
		if(PIND&b00010000) {
			// button probably released
			pressedCounter = 0;
			if(pressed){
				if(releasedCounter > 5){
					// button released for sure
					pressed = 0;
				} else {
					releasedCounter++;
				}
			}
		} else {
			// button probably pressed
			releasedCounter = 0;
			if(!pressed){
				if(pressedCounter > 5){
					// button pressed for sure
					pressed = 1;
					if(redCable){
						// if red wire is active
						if(minute > 0){
							// subtract 1 minute
							minute--;
							segment[0] = minute;
						} else if(second > 10){
							// subtract 10 seconds
							second-=10;
							segment[1] = second/10;
							segment[2] = second%10;
						} /*else if (!counting && restart == 255){
							// reset countdown after some time after reaching absolute zero
							minute = 9;
							second = 59;
							frames = 24;
							segment[0] = minute;
							segment[1] = second/10;
							segment[2] = second%10;
							segment[3] = frames/10;
							segment[4] = frames%10;
							restart = 0;
							OCR1A = 19999;
							TCNT1 = 0;
							counting = 1;
						}*/
					} else {
						// unpause countdown after cutting red cable
						// redCable = 1;
					}
				} else {
					pressedCounter++;
				}
			}
		}

		// check red cable
		if(redCable > 1){
			if(PINC&b00100000){
				// red cable probably cut
				redCable++;
				if(redCable > 20)
					// red cable cut for sure -> stop countdown
					redCable = 0;
			}
		}

		// check blue cable
		if(blueCable > 1 && redCable && counting){
			if(PINC&b00010000){
				// blue cable probably cut
				if(blueCable > 20){
					// blue cable cut for sure -> set last 2 seconds
					minute = 0;
					blueCable = 0;
					if(!redCable)
						redCable = 1;
					if(second > 1){
						second = 1;
						frames = 24;
						//OCR1A = 30000;
					}
					segment[0] = minute;
					segment[1] = second/10;
					segment[2] = second%10;
					segment[3] = frames/10;
					segment[4] = frames%10;
				} else {
					blueCable++;
				}
			}
		}

	}

	return 0;
}
